using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.BusinessLogicsContracts;
using BlacksmithWorkshopContracts.SearchModels;
using BlacksmithWorkshopContracts.StoragesContracts;
using BlacksmithWorkshopContracts.ViewModels;
using BlacksmithWorkshopDataModels.Models;
using Microsoft.Extensions.Logging;

namespace BlacksmithWorkshopBusinessLogic.BusinessLogics
{
    public class ShopLogic : IShopLogic
	{
		private readonly ILogger _logger;
		private readonly IShopStorage _shopStorage;
		public ShopLogic(ILogger<ShopLogic> logger, IShopStorage ShopStorage)
		{
			_logger = logger;
			_shopStorage = ShopStorage;
		}
		public List<ShopViewModel>? ReadList(ShopSearchModel? model)
		{
			_logger.LogInformation("ReadList. ShopName:{ShopName}. Id:{ Id}", model?.ShopName, model?.Id);
			var list = model == null ? _shopStorage.GetFullList() :
			_shopStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count:{Count}", list.Count);
			return list;
		}
		public ShopViewModel? ReadElement(ShopSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. ShopName:{ShopName}.Id:{ Id}", model.ShopName, model.Id);
			var element = _shopStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
			return element;
		}
		public bool Create(ShopBindingModel model)
		{
			CheckModel(model);
			if (_shopStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}
		public bool Update(ShopBindingModel model)
		{
			CheckModel(model);
			if (_shopStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		public bool Delete(ShopBindingModel model)
		{
			CheckModel(model, false);
			_logger.LogInformation("Delete. Id:{Id}", model.Id);
			if (_shopStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}
		public bool ReplenishManufactures(ShopSearchModel model, IManufactureModel manufacture, int count)
		{
			_logger.LogInformation("Try to replenish manufactures. ShopName:{ShopName}. Id:{Id}", model.ShopName, model.Id);
			if (model == null)
            {
                _logger.LogWarning("Read operation failed");
                throw new ArgumentNullException(nameof(model));
			}
			ShopViewModel? curModel = ReadElement(model);
			if (curModel == null)
			{
				_logger.LogWarning("Read operation failed");
				throw new ArgumentNullException(nameof(curModel));
			}
			if (manufacture == null)
            {
                _logger.LogWarning("Read operation failed");
                throw new ArgumentNullException(nameof(manufacture));
			}
            if (count <= 0)
            {
                _logger.LogWarning("Read operation failed");
                throw new ArgumentException("Количество должно быть положительным");
			}
            int countItems = curModel.ShopManufactures.Select(x => x.Value.Item2).Sum();
            if (curModel.MaxCapacity - countItems >= count)
			{
                if (curModel.ShopManufactures.TryGetValue(manufacture.Id, out var sameDocument))
                {
                    curModel.ShopManufactures[manufacture.Id] = (manufacture, sameDocument.Item2 + count);
                    _logger.LogInformation("Same manufacture found by supply. Added {0} of {1} in {2} shop", count, manufacture.ManufactureName, curModel.ShopName);
                }
                else
                {
                    curModel.ShopManufactures[manufacture.Id] = (manufacture, count);
                    _logger.LogInformation("New manufacture added by supply. Added {0} of {1} in {2} shop", count, manufacture.ManufactureName, curModel.ShopName);
                }
                _shopStorage.Update(new()
                {
                    Id = curModel.Id,
                    ShopName = curModel.ShopName,
                    Address = curModel.Address,
                    OpeningDate = curModel.OpeningDate,
                    ShopManufactures = curModel.ShopManufactures,
                    MaxCapacity = curModel.MaxCapacity
                });
            }
			else
			{
                _logger.LogWarning("Required shop is overflowed");
                return false;
            }
			return true;
		}
		private void CheckModel(ShopBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			if (string.IsNullOrEmpty(model.ShopName))
			{
				throw new ArgumentNullException("Нет названия магазина", nameof(model.ShopName));
			}
			if (string.IsNullOrEmpty(model.Address))
			{
				throw new ArgumentNullException("Нет адреса магазина", nameof(model.Address));
			}
			_logger.LogInformation("Shop. ShopName:{ShopName}.Address:{Address}. DateOpen:{DateOpen}. Id: { Id}", 
			model.ShopName, model.Address, model.OpeningDate, model.Id);
			var element = _shopStorage.GetElement(new ShopSearchModel
			{
				ShopName = model.ShopName
			});
			if (element != null && element.Id != model.Id)
			{
				throw new InvalidOperationException("Магазин с таким названием уже есть");
			}
		}
        public bool SellManufactures(IManufactureModel manufacture, int count)
        {
            return _shopStorage.SellManufactures(manufacture, count);
        }
    }
}
